/****************************************************************************
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** Permission is hereby granted, free of charge, to any person obtaining a copy
** of this software and associated documentation files (the "Software"), to deal
** in the Software without restriction, including without limitation the rights
** to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
** copies of the Software, and to permit persons to whom the Software is
** furnished to do so, subject to the following conditions:
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
**  USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

/*!
 * @file lteiot12.h
 * @brief This file contains API for LTE IoT 12 Click Driver.
 */

#ifndef LTEIOT12_H
#define LTEIOT12_H

#ifdef __cplusplus
extern "C"{
#endif

#include "mikrosdk_version.h"

#ifdef __GNUC__
#if mikroSDK_GET_VERSION < 20800ul
#include "rcu_delays.h"
#else
#include "delays.h"
#endif
#endif

#include "drv_digital_out.h"
#include "drv_digital_in.h"
#include "drv_uart.h"

/*!
 * @addtogroup lteiot12 LTE IoT 12 Click Driver
 * @brief API for configuring and manipulating LTE IoT 12 Click driver.
 * @{
 */

/**
 * @defgroup lteiot12_cmd LTE IoT 12 Device Settings
 * @brief Settings for registers of LTE IoT 12 Click driver.
 */

/**
 * @addtogroup lteiot12_cmd
 * @{
 */

/**
 * @brief LTE IoT 12 control commands.
 * @details Specified setting for control commands of LTE IoT 12 Click driver.
 */
#define LTEIOT12_CMD_AT                             "AT"
#define LTEIOT12_CMD_ATI                            "ATI"
#define LTEIOT12_CMD_CGMR                           "AT+CGMR"
#define LTEIOT12_CMD_CFUN                           "AT+CFUN"
#define LTEIOT12_CMD_CREG                           "AT+CREG"
#define LTEIOT12_CMD_CEREG                          "AT+CEREG"
#define LTEIOT12_CMD_CGREG                          "AT+CGREG"
#define LTEIOT12_CMD_CGDCONT                        "AT+CGDCONT"
#define LTEIOT12_CMD_CMGF                           "AT+CMGF"
#define LTEIOT12_CMD_CGDCONT                        "AT+CGDCONT"
#define LTEIOT12_CMD_CIMI                           "AT+CIMI"
#define LTEIOT12_CMD_CGATT                          "AT+CGATT"
#define LTEIOT12_CMD_CSQ                            "AT+CSQ"
#define LTEIOT12_CMD_CESQ                           "AT+CESQ"
#define LTEIOT12_CMD_COPS                           "AT+COPS"
#define LTEIOT12_CMD_CSCON                          "AT+CSCON"
#define LTEIOT12_CMD_CMEE                           "AT+CMEE"
#define LTEIOT12_CMD_CMGS                           "AT+CMGS"
#define LTEIOT12_CMD_CGACT                          "AT+CGACT"
#define LTEIOT12_CMD_CSCS                           "AT+CSCS"
#define LTEIOT12_CMD_CMGF                           "AT+CMGF"
#define LTEIOT12_CMD_CPSMS                          "AT+CPSMS"
#define LTEIOT12_CMD_SM                             "AT+SM"
#define LTEIOT12_CMD_QSCLK                          "AT+QSCLK"
#define LTEIOT12_CMD_QIOPEN                         "AT+QIOPEN"
#define LTEIOT12_CMD_QISEND                         "AT+QISEND"
#define LTEIOT12_CMD_QIRD                           "AT+QIRD"
#define LTEIOT12_CMD_QICLOSE                        "AT+QICLOSE"
#define LTEIOT12_CMD_QCSCON                         "AT+QCSCON"
#define LTEIOT12_CMD_QGPS                           "AT+QGPS"
#define LTEIOT12_CMD_QGPSCFG                        "AT+QGPSCFG"
#define LTEIOT12_CMD_QGPSGNMEA                      "AT+QGPSGNMEA"
#define LTEIOT12_CMD_QGPSLOC                        "AT+QGPSLOC"

/**
 * @brief LTE IoT 12 response.
 * @details Specified response settings of LTE IoT 12 Click driver.
 */
#define LTEIOT12_RSP_OK                             "OK"
#define LTEIOT12_RSP_RDY                            "RDY"
#define LTEIOT12_RSP_APPRDY                         "APP RDY"
#define LTEIOT12_RSP_ERROR                          "ERROR"
#define LTEIOT12_RSP_START                          "$"
#define LTEIOT12_RSP_DELIMITER                      ","
#define LTEIOT12_RSP_GPGGA                          "GPGGA"

/**
 * @brief LTE IoT 12 NMEA command elements.
 * @details Specified NMEA command elements of LTE IoT 12 Click driver.
 */
#define LTEIOT12_NMEA_GNGGA                         1
#define LTEIOT12_NMEA_GPGLL                         3
#define LTEIOT12_NMEA_GNGGA_NUM_ELEMENT             15
#define LTEIOT12_NMEA_GPGLL_NUM_ELEMENT             5

/**
 * @brief LTE IoT 12 GNGGA command elements.
 * @details Specified GNGGA command elements of LTE IoT 12 Click driver.
 */
#define LTEIOT12_GNGGA_TIME                         1 
#define LTEIOT12_GNGGA_LATITUDE                     2
#define LTEIOT12_GNGGA_LATITUDE_SIDE                3
#define LTEIOT12_GNGGA_LONGITUDE                    4
#define LTEIOT12_GNGGA_LONGITUDE_SIDE               5
#define LTEIOT12_GNGGA_QUALITY_INDICATOR            6
#define LTEIOT12_GNGGA_NUMBER_OF_SATELLITES         7
#define LTEIOT12_GNGGA_H_DILUTION_OF_POS            8
#define LTEIOT12_GNGGA_ALTITUDE                     9
#define LTEIOT12_GNGGA_ALTITUDE_UNIT                10
#define LTEIOT12_GNGGA_GEOIDAL_SEPARATION           11
#define LTEIOT12_GNGGA_GEOIDAL_SEPARATION_UNIT      12
#define LTEIOT12_GNGGA_TIME_SINCE_LAST_DGPS         13
#define LTEIOT12_GNGGA_DGPS_REFERENCE_STATION_ID    14

/**
 * @brief LTE IoT 12 pin state.
 * @details Specified pin state of LTE IoT 12 Click driver.
 */
#define LTEIOT12_PIN_STATE_HIGH                     0x01
#define LTEIOT12_PIN_STATE_LOW                      0x00

/**
 * @brief LTE IoT 12 driver buffer size.
 * @details Specified size of driver ring buffer.
 * @note Increase buffer size if needed.
 */
#define LTEIOT12_TX_DRV_BUFFER_SIZE                 300
#define LTEIOT12_RX_DRV_BUFFER_SIZE                 300

/*! @} */ // lteiot12_cmd

/**
 * @defgroup lteiot12_map LTE IoT 12 MikroBUS Map
 * @brief MikroBUS pin mapping of LTE IoT 12 Click driver.
 */

/**
 * @addtogroup lteiot12_map
 * @{
 */

/**
 * @brief MikroBUS pin mapping.
 * @details Mapping pins of LTE IoT 12 Click to the selected MikroBUS.
 */
#define LTEIOT12_MAP_MIKROBUS( cfg, mikrobus ) \
    cfg.tx_pin  = MIKROBUS( mikrobus, MIKROBUS_TX ); \
    cfg.rx_pin  = MIKROBUS( mikrobus, MIKROBUS_RX ); \
    cfg.sta = MIKROBUS( mikrobus, MIKROBUS_AN ); \
    cfg.pwr = MIKROBUS( mikrobus, MIKROBUS_RST ); \
    cfg.rts = MIKROBUS( mikrobus, MIKROBUS_CS ); \
    cfg.ri  = MIKROBUS( mikrobus, MIKROBUS_PWM ); \
    cfg.cts = MIKROBUS( mikrobus, MIKROBUS_INT );

/*! @} */ // lteiot12_map
/*! @} */ // lteiot12

/**
 * @brief LTE IoT 12 Click context object.
 * @details Context object definition of LTE IoT 12 Click driver.
 */
typedef struct
{
    // Output pins
    digital_out_t pwr;                          /**< Power pin. */
    digital_out_t rts;                          /**< Ready to send. */

    // Input pins
    digital_in_t sta;                           /**< Module?s operation status. */
    digital_in_t ri;                            /**< UART Ring indication. */
    digital_in_t cts;                           /**< Clear to send. */

    // Modules
    uart_t uart;                                /**< UART driver object. */

    // Buffers
    uint8_t uart_rx_buffer[ LTEIOT12_RX_DRV_BUFFER_SIZE ];  /**< Buffer size. */
    uint8_t uart_tx_buffer[ LTEIOT12_TX_DRV_BUFFER_SIZE ];  /**< Buffer size. */

} lteiot12_t;

/**
 * @brief LTE IoT 12 Click configuration object.
 * @details Configuration object definition of LTE IoT 12 Click driver.
 */
typedef struct
{
    // Communication gpio pins
    pin_name_t rx_pin;                          /**< RX pin. */
    pin_name_t tx_pin;                          /**< TX pin. */

    // Additional gpio pins
    pin_name_t sta;                             /**< Module?s operation status. */
    pin_name_t pwr;                             /**< Power pin. */
    pin_name_t rts;                             /**< Ready to send. */
    pin_name_t ri;                              /**< UART Ring indication. */
    pin_name_t cts;                             /**< Clear to send. */

    // Static variable
    uint32_t         baud_rate;                 /**< Clock speed. */
    bool             uart_blocking;             /**< Wait for interrupt or not. */
    uart_data_bits_t data_bit;                  /**< Data bits. */
    uart_parity_t    parity_bit;                /**< Parity bit. */
    uart_stop_bits_t stop_bit;                  /**< Stop bits. */

} lteiot12_cfg_t;

/**
 * @brief LTE IoT 12 Click return value data.
 * @details Predefined enum values for driver return values.
 */
typedef enum
{
    LTEIOT12_OK = 0,
    LTEIOT12_ERROR = -1,
    LTEIOT12_OWERFLOW = -2,
    LTEIOT12_TIMEOUT = -3

} lteiot12_return_value_t;

/*!
 * @addtogroup lteiot12 LTE IoT 12 Click Driver
 * @brief API for configuring and manipulating LTE IoT 12 Click driver.
 * @{
 */

/**
 * @brief LTE IoT 12 configuration object setup function.
 * @details This function initializes click configuration structure to initial
 * values.
 * @param[out] cfg : Click configuration structure.
 * See #lteiot12_cfg_t object definition for detailed explanation.
 * @return Nothing.
 * @note The all used pins will be set to unconnected state.
 */
void lteiot12_cfg_setup ( lteiot12_cfg_t *cfg );

/**
 * @brief LTE IoT 12 initialization function.
 * @details This function initializes all necessary pins and peripherals used
 * for this click board.
 * @param[out] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] cfg : Click configuration structure.
 * See #lteiot12_cfg_t object definition for detailed explanation.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
err_t lteiot12_init ( lteiot12_t *ctx, lteiot12_cfg_t *cfg );

/**
 * @brief LTE IoT 12 data writing function.
 * @details This function writes a desired number of data bytes by using UART serial interface.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] data_in : Data buffer for sending.
 * @param[in] len : Number of bytes for sending.
 * @return @li @c  >=0 - Success,
 *         @li @c   <0 - Error.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
err_t lteiot12_generic_write ( lteiot12_t *ctx, uint8_t *data_in, uint16_t len );

/**
 * @brief LTE IoT 12 data reading function.
 * @details This function reads a desired number of data bytes by using UART serial interface.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[out] data_out : Output read data.
 * @param[in] len : Number of bytes to be read.
 * @return @li @c  >0 - Number of data bytes read,
 *         @li @c <=0 - Error/Empty Ring buffer.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
err_t lteiot12_generic_read ( lteiot12_t *ctx, uint8_t *data_out, uint16_t len );

/**
 * @brief LTE IoT 12 set PWR pin function.
 * @details This function sets the PWR pin logic state.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] pin_state : Pin logic state.
 * @return None.
 * @note None.
 */
void lteiot12_set_pwr_pin ( lteiot12_t *ctx, uint8_t pin_state );

/**
 * @brief LTE IoT 12 set RTS pin function.
 * @details This function sets the RTS pin logic state.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] pin_state : Pin logic state.
 * @return None.
 * @note None.
 */
void lteiot12_set_rts_pin ( lteiot12_t *ctx, uint8_t pin_state );

/**
 * @brief LTE IoT 12 get cts pin function.
 * @details This function returns the clear to send (CTS) pin logic state.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @return Pin logic state.
 * @note None.
 */
uint8_t lteiot12_get_cts_pin ( lteiot12_t *ctx );

/**
 * @brief LTE IoT 12 hardware reset function.
 * @details This function is used to perform hardware reset.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @return Nothing.
 * @note None.
 */
void lteiot12_hw_reset ( lteiot12_t *ctx );

/**
 * @brief LTE IoT 12 send command function.
 * @details This function sends a specified command to the LTE IoT 12 Click module.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] cmd  Command variable.
 * @return Nothing.
 * @note None.
 */
void lteiot12_send_cmd ( lteiot12_t *ctx, uint8_t *cmd );

/**
 * @brief LTE IoT 12 send command function with parameter.
 * @details This function sends a command with specified parameter to the click module.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] at_cmd_buf  Command buffer.
 * @param[in] param_buf  Parameter buffer.
 * @return Nothing.
 * @note None.
 */
void lteiot12_send_cmd_with_params ( lteiot12_t *ctx, uint8_t *at_cmd_buf, uint8_t *param_buf );

/**
 * @brief LTE IoT 12 check the sent command.
 * @details This function checks the command that is sent.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] at_cmd_buf  Command buffer.
 * @return Nothing.
 * @note None.
 */
void lteiot12_send_cmd_check ( lteiot12_t *ctx, uint8_t *at_cmd_buf );

/**
 * @brief LTE IoT 12 check the command parameters.
 * @details This function checks the command that is sent.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] at_cmd_buf  Command buffer.
 * @return Nothing.
 * @note None.
 */
void lteiot12_send_cmd_params_check ( lteiot12_t *ctx, uint8_t *at_cmd_buf );

/**
 * @brief LTE IoT 12 set sim card APN.
 * @details This function sets APN for sim card.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] sim_apn  SIM card APN.
 * @return Nothing.
 * @note None.
 */
void lteiot12_set_sim_apn ( lteiot12_t *ctx, uint8_t *sim_apn );

/**
 * @brief LTE IoT 12 send SMS in text mode.
 * @details This function sends text message to a phone number.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] phone_number : Phone number to message.
 * @param[in] sms_text : Message to be sent.
 * @return Nothing.
 * @note None.
 */
void lteiot12_send_sms_text_mode ( lteiot12_t *ctx, uint8_t *phone_number, uint8_t *sms_text );

/**
 * @brief LTE IoT 12 send SMS in PDU mode.
 * @details This function sends text message to a phone number in PDU mode.
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @param[in] service_center_number : SMSC of the SIM card.
 * @param[in] phone_number : Phone number to message.
 * @param[in] sms_text  Message to be sent.
 * @return @li @c  >=0 - Success,
 *         @li @c   <0 - Error.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
err_t lteiot12_send_sms_pdu ( lteiot12_t *ctx, uint8_t *service_center_number, uint8_t *phone_number, uint8_t *sms_text );

/**
 * @brief LTE IoT 12 parse GNGGA function.
 * @details This function parses the GNGGA data from the read response buffer.
 * @param[in] rsp_buf : Response buffer.
 * @param[in] gngga_element : GNGGA element position [1-14].
 * @param[out] element_data : Element data parsed.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
err_t lteiot12_parse_gngga ( uint8_t *rsp_buf, uint8_t gngga_element, uint8_t *element_data );

#ifdef __cplusplus
}
#endif
#endif // LTEIOT12_H

/*! @} */ // lteiot12

// ------------------------------------------------------------------------ END
