/*!
 * @file main.c
 * @brief LTE IoT 12 Click Example.
 *
 * # Description
 * Application example shows device capability of connecting to the network and 
 * sending SMS, TCP/UDP messages or getting GNSS data using standard "AT" commands. 
 *
 * The demo application is composed of two sections :
 *
 * ## Application Init
 * Sets the device configuration for sending SMS, TCP/UDP messages or getting GNSS data.
 *
 * ## Application Task
 * Depending on the selected demo example, it sends an SMS message 
 * (in PDU or TXT mode) or a TCP/UDP message or getting GNSS data ( position and altitude ).
 *
 * ## Additional Function
 * - static void lteiot12_clear_app_buf ( void )
 * - static err_t lteiot12_process ( lteiot12_t *ctx )
 * - static err_t lteiot12_rsp_check ( lteiot12_t *ctx, uint8_t *response )
 * - static void lteiot12_error_check( err_t error_flag )
 * - static void lteiot12_config_for_example ( void )
 * - static void lteiot12_example ( void )
 * - static void lteiot12_check_connection ( void )
 * - static void lteiot12_log_response ( void )
 * - static void lteiot12_error_check ( err_t error_flag )
 * - static err_t lteiot12_check_response ( void )
 * - static void lteiot12_check_con_to_network ( void )
 * - static void gnss_parser_application ( uint8_t *rsp )
 *
 * @note
 * In order for the examples to work, user needs to set the APN and SMSC (SMS PDU mode only)
 * of entered SIM card as well as the phone number (SMS mode only) to which he wants to send an SMS.
 * Enter valid values for the following macros: SIM_APN, SIM_SMSC and PHONE_NUMBER_TO_MESSAGE.
 * Example:
 *   SIM_APN "internet"
 *   SIM_SMSC "+381610401"
 *   PHONE_NUMBER_TO_MESSAGE "+381659999999"
 * 
 * @author Stefan Ilic
 * 
 */

#include "board.h"
#include "log.h"
#include "lteiot12.h"
#include "conversions.h"

#define PROCESS_BUFFER_SIZE 300

// Example selection macros
#define TCP_UDP_EXAMPLE                     0               // Example of sending messages to a TCP/UDP echo server
#define SMS_EXAMPLE                         1               // Example of sending SMS to a phone number
#define GNSS_EXAMPLE                        2               // Example of getting GNSS data
#define DEMO_EXAMPLE                        TCP_UDP_EXAMPLE // Example selection macro

// SMS example parameters
#define SIM_APN                             "internet"      // Set valid SIM APN 
#define SIM_SMSC                            ""              // Set valid SMS Service Center Address - only in PDU mode 
#define PHONE_NUMBER_TO_MESSAGE             ""              // Set Phone number to message 
#define SMS_MODE                            "1"             // SMS mode: "0" - PDU, "1" - TXT

// TCP/UDP example parameters
#define REMOTE_IP                           "77.46.162.162" // TCP/UDP echo server IP address
#define REMOTE_PORT                         "51111"         // TCP/UDP echo server port

#define MESSAGE_CONTENT                     "LTE IoT 12 Click Example"   // Message content

/**
 * @brief Example states.
 * @details Predefined enum values for application example state.
 */
typedef enum
{
    LTEIOT12_CONFIGURE_FOR_EXAMPLE = 1,
    LTEIOT12_EXAMPLE

} ltelte12_example_state_t;

static lteiot12_t lteiot12;
static log_t logger;

static uint8_t app_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
static int32_t app_buf_len = 0;
static err_t app_error_flag;
static ltelte12_example_state_t app_connection_status = LTEIOT12_CONFIGURE_FOR_EXAMPLE;
static uint8_t gps_info_message[ 200 ] = { 0 };

/**
 * @brief LTE IoT 12 clearing application buffer.
 * @details This function clears memory of application buffer and reset its length.
 * @note None.
 */
static void lteiot12_clear_app_buf ( void );

/**
 * @brief LTE IoT 12 data reading function.
 * @details This function reads data from device and concatenates data to application buffer. 
 * @param[in] ctx : Click context object.
 * See #lteiot12_t object definition for detailed explanation.
 * @return @li @c  0 - Read some data.
 *         @li @c -1 - Nothing is read.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t lteiot12_process ( lteiot12_t *ctx );

/**
 * @brief LTE IoT 12 response check.
 * @details This function checks for response and returns the status of response.
 * @return application status.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t lteiot12_rsp_check ( lteiot12_t *ctx, uint8_t *response );

/**
 * @brief LTE IoT 12 check for errors.
 * @details This function checks for different types of errors and logs them on UART.
 * @note None.
 */
static void lteiot12_error_check( err_t error_flag );

/**
 * @brief Configure device for example.
 * @details Configure device for sending messages or sending messages to TCP/UDP Echo server, or GNSS example.
 * @return Nothing.
 * @note None.
 */
static void lteiot12_config_for_example ( void );

/**
 * @brief Execute example.
 * @details This function executes SMS, TCP/UDP or GNSS example depending on the DEMO_EXAMPLE macro.
 * @return Nothing.
 * @note None.
 */
static void lteiot12_example ( void );

/**
 * @brief Wait for connection signal.
 * @details Wait for connection signal from CREG URC.
 * @return Nothing.
 * @note None.
 */
static void lteiot12_check_connection ( void );

/**
 * @brief Logs application buffer.
 * @details This function logs data from application buffer.
 * @note None.
 */
static void lteiot12_log_response ( void );

/**
 * @brief Check for errors.
 * @details This function checks for different types of 
 * errors and logs them on UART or logs the response if no errors occured.
 * @note None.
 */
static void lteiot12_error_check ( err_t error_flag );

/**
 * @brief LTE IoT 12 response read function.
 * @details This function reads the response of the sent command and writes it on the USB UART.
 * @return @li @c  0 - Response OK.
 *         @li @c -1 - Response ERROR.
 * See #err_t definition for detailed explanation.
 * @note None.
 */
static err_t lteiot12_check_response ( void );

/**
 * @brief Check for connection to the network.
 * @details Sends commands to check connection to the specified network.
 * @return Nothing.
 * @note None.
 */
static void lteiot12_check_con_to_network ( void );

/**
 * @brief GNSS parser application.
 * @details This function logs GNSS data on the USB UART terminal..
 * @param rsp Response buffer.
 * @note None.
 */
static void gnss_parser_application ( uint8_t *rsp );

void application_init ( void ) 
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    lteiot12_cfg_t lteiot12_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    lteiot12_cfg_setup( &lteiot12_cfg );
    LTEIOT12_MAP_MIKROBUS( lteiot12_cfg, MIKROBUS_1 );
    if ( UART_ERROR == lteiot12_init( &lteiot12, &lteiot12_cfg ) ) 
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    log_printf( &logger, " Power on device... \r\n" );
    lteiot12_hw_reset( &lteiot12 );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_RDY );
    #define LTEIOT12_MIN_FUN                            "0"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CFUN, LTEIOT12_MIN_FUN );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_APPRDY );
    lteiot12_error_check( app_error_flag );

    lteiot12_send_cmd( &lteiot12, LTEIOT12_CMD_AT );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );
    
    lteiot12_send_cmd( &lteiot12, LTEIOT12_CMD_ATI );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );

    #define LTEIOT12_FULL_FUN                           "1"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CFUN, LTEIOT12_FULL_FUN );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );
    
    #define LTEIOT12_DISABLE_PSM                        "1"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CPSMS, LTEIOT12_DISABLE_PSM );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );
    
    #define LTEIOT12_EN_LOCATION_AND_NETWORK            "2"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CEREG, LTEIOT12_EN_LOCATION_AND_NETWORK );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );     

    #define LTEIOT12_DISABLE_SLEEP                      "0"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QSCLK, LTEIOT12_DISABLE_SLEEP );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  
    
    
    lteiot12_send_cmd( &lteiot12, LTEIOT12_CMD_CIMI );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag ); 
    
    log_info( &logger, " Application Task " );
    
}

void application_task ( void ) 
{
    switch( app_connection_status )
    {
        case LTEIOT12_CONFIGURE_FOR_EXAMPLE:
        {
            lteiot12_config_for_example( );
            break;
        }
        case LTEIOT12_EXAMPLE:
        {
            lteiot12_example ( );
            break;
        }
        default:
        {
            log_error( &logger, "Application status error! " );
            app_connection_status = LTEIOT12_CONFIGURE_FOR_EXAMPLE;
            Delay_ms( 1000 );
            break;
        }
    }
    
}

int main ( void ) 
{
    application_init( );

    for ( ; ; ) 
    {
        application_task( );
    }

    return 0;
}

static void lteiot12_clear_app_buf ( void ) 
{
    memset( app_buf, 0, app_buf_len );
    app_buf_len = 0;
}

static err_t lteiot12_process ( lteiot12_t *ctx ) 
{
    uint8_t rx_buf[ PROCESS_BUFFER_SIZE ] = { 0 };
    int32_t rx_size = 0;
    rx_size = lteiot12_generic_read( ctx, rx_buf, PROCESS_BUFFER_SIZE );
    if ( rx_size > 0 ) 
    {
        int32_t buf_cnt = app_buf_len;
        if ( ( ( app_buf_len + rx_size ) > PROCESS_BUFFER_SIZE ) && ( app_buf_len > 0 ) ) 
        {
            buf_cnt = PROCESS_BUFFER_SIZE - ( ( app_buf_len + rx_size ) - PROCESS_BUFFER_SIZE );
            memmove ( app_buf, &app_buf[ PROCESS_BUFFER_SIZE - buf_cnt ], buf_cnt );
        }
        for ( int32_t rx_cnt = 0; rx_cnt < rx_size; rx_cnt++ ) 
        {
            if ( rx_buf[ rx_cnt ] ) 
            {
                app_buf[ buf_cnt++ ] = rx_buf[ rx_cnt ];
                if ( app_buf_len < PROCESS_BUFFER_SIZE )
                {
                    app_buf_len++;
                }
            }
        }
        return LTEIOT12_OK;
    }
    return LTEIOT12_ERROR;
}

static err_t lteiot12_rsp_check ( lteiot12_t *ctx, uint8_t *response ) 
{
    uint16_t timeout_cnt = 0;
    uint16_t timeout = 50000;
    
    err_t error_flag = lteiot12_process( ctx );
    if ( ( error_flag != 0 ) && ( error_flag != -1 ) ) 
    {
        return error_flag;
    }
    
    while ( ( strstr( app_buf, response ) == 0 ) && ( strstr( app_buf, LTEIOT12_RSP_ERROR ) == 0 ) ) 
    {
        error_flag = lteiot12_process( ctx );
        if ( ( error_flag != 0 ) && ( error_flag != -1 ) ) 
        {
            return error_flag;
        }
        
        timeout_cnt++;
        if ( timeout_cnt > timeout ) 
        {
            return LTEIOT12_TIMEOUT;
        }
        
        Delay_ms( 1 );
    }
    
    for ( int32_t buf_cnt = 0; buf_cnt < app_buf_len; buf_cnt++ ) 
    {
        log_printf( &logger, "%c", app_buf[ buf_cnt ] );
    }
    log_printf( &logger, "\r\n" );
    lteiot12_check_connection( );
    lteiot12_clear_app_buf( );
    Delay_ms( 500 );
    
    
    log_printf( &logger, "-----------------------------------\r\n" );
}

static void lteiot12_error_check( err_t error_flag ) 
{
    if ( ( error_flag != 0 ) && ( error_flag != LTEIOT12_ERROR ) ) 
    {
        switch ( error_flag ) 
        {
            case LTEIOT12_OWERFLOW: 
            {
                log_error( &logger, " Overflow!" );
                break;
            }
            case LTEIOT12_TIMEOUT:
            {
                log_error( &logger, " Timeout!" );
                break;
            }
            default: 
            {
                break;
            }
        }
    }
}

static void lteiot12_config_for_example ( void )
{
#if ( DEMO_EXAMPLE == TCP_UDP_EXAMPLE )
    #define LTEIOT12_PS_ATTACHED                        "1"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CGATT, LTEIOT12_PS_ATTACHED );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  
    
    if ( LTEIOT12_OK == app_error_flag )
    {
        app_connection_status = LTEIOT12_EXAMPLE;
    }
    #define LTEIOT12_URC_ENABLE                         "1"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QCSCON, LTEIOT12_URC_ENABLE );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag ); 
#elif ( ( DEMO_EXAMPLE == SMS_EXAMPLE ) || ( DEMO_EXAMPLE == GNSS_EXAMPLE ) )
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CMGF, SMS_MODE );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    #define LTEIOT12_CHAR_SET_GMS                       "\"GSM\""    
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CSCS, LTEIOT12_CHAR_SET_GMS );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    lteiot12_send_cmd( &lteiot12, LTEIOT12_CMD_CIMI );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    lteiot12_set_sim_apn( &lteiot12, SIM_APN );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK );
    lteiot12_error_check( app_error_flag );

    #define LTEIOT12_EN_LOCATION_AND_NETWORK            "2"
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_CREG, LTEIOT12_EN_LOCATION_AND_NETWORK );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag ); 

    #define LTEIOT12_TURN_ON_GNSS                       "1"   
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QGPS, LTEIOT12_TURN_ON_GNSS );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );
    
    #define LTEIOT12_CONFIG_GNSS                        "\"nmeasrc\",1"  
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QGPSCFG, LTEIOT12_CONFIG_GNSS );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK );
    lteiot12_error_check( app_error_flag );
    for( ; ; )
    {
        lteiot12_check_con_to_network( );
        if ( LTEIOT12_CONFIGURE_FOR_EXAMPLE != app_connection_status )
        {
            break;
        }
    }
#endif 
}

static void lteiot12_example ( void )
{
#if ( DEMO_EXAMPLE == TCP_UDP_EXAMPLE )
    #define CONTEXTID                                   "1"
    #define TCP_PROTOCOL                                "0"
    #define UDP_PROTOCOL                                "1"
    #define URC_READ_SOCKET_DATA                        "+QIURC: \"recv\""
    #define TCP                                         "TCP"
    #define UDP                                         "UDP"
    
    uint8_t cmd_buf[ 100 ] = { 0 };
    uint16_t timeout_cnt = 0;
    uint16_t timeout = 30000;
    
    // Get message length
    uint8_t message_len_buf[ 5 ] = { 0 };
    uint16_t message_len = strlen( MESSAGE_CONTENT );
    uint16_to_str( message_len, message_len_buf );
    l_trim( message_len_buf );
    r_trim( message_len_buf );
    
    // Create TCP socket
    strcpy( cmd_buf, CONTEXTID );
    strcat( cmd_buf, "," );
    strcat( cmd_buf, TCP_PROTOCOL );
    strcat( cmd_buf, ",\"" );
    strcat( cmd_buf, TCP );
    strcat( cmd_buf, "\"" );
    strcat( cmd_buf, ",\"" );
    strcat( cmd_buf, REMOTE_IP );
    strcat( cmd_buf, "\"," ); 
    strcat( cmd_buf, REMOTE_PORT ); 
    
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QIOPEN, cmd_buf );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  
    Delay_ms( 2000 );
    
    #define LTEIOT12_TCP_SOCKET_OPEN                "QIOPEN: 0,0"
    lteiot12_rsp_check( &lteiot12, LTEIOT12_TCP_SOCKET_OPEN );
    
    if ( LTEIOT12_OK == app_error_flag )
    {
        log_info( &logger, "SENDING MESSAGES TO TCP" );
        // Write message to TCP socket
        strcpy( cmd_buf, TCP_PROTOCOL );
        strcat( cmd_buf, "," );
        strcat( cmd_buf, message_len_buf );
        lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QISEND, cmd_buf );
        lteiot12_rsp_check( &lteiot12, ">"  );
        strcpy( cmd_buf, MESSAGE_CONTENT );
        strcat( cmd_buf, "\032" );
        
        lteiot12_send_cmd( &lteiot12, cmd_buf );
        
        lteiot12_rsp_check( &lteiot12, "SEND OK"  );
    
        // Read response message from TCP socket
        for ( ; ; )
        {
            lteiot12_process( &lteiot12 );
            uint8_t * __generic_ptr start_response_buf = strstr( app_buf, URC_READ_SOCKET_DATA );
            if ( start_response_buf )
            {
                log_info( &logger, "RECEIVED MESSAGE" );
                strcpy( cmd_buf, TCP_PROTOCOL );
                strcat( cmd_buf, "," );
                strcat( cmd_buf, "256" );
                lteiot12_clear_app_buf( );
                lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QIRD, cmd_buf );
                app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
                lteiot12_error_check( app_error_flag );  
            
                if ( LTEIOT12_OK == app_error_flag )
                {
                    break;
                }
            }
            if ( timeout_cnt++ > timeout )
            {
                log_error( &logger, "TIMEOUT!!!" );
                break;
            }   
            Delay_ms( 1 );
        }
        timeout_cnt = 0;
        Delay_ms( 2000 );
        
        // Close TCP socket
        lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QICLOSE, TCP_PROTOCOL );
        app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
        lteiot12_error_check( app_error_flag );  
    }
    else
    {
        log_error( &logger, "Not connected to the TCP server" );
    }
    
    // Create UDP socket
    strcpy( cmd_buf, CONTEXTID );
    strcat( cmd_buf, "," );
    strcat( cmd_buf, UDP_PROTOCOL );
    strcat( cmd_buf, ",\"" );
    strcat( cmd_buf, UDP );
    strcat( cmd_buf, "\"" );
    strcat( cmd_buf, ",\"" );
    strcat( cmd_buf, REMOTE_IP );
    strcat( cmd_buf, "\"," ); 
    strcat( cmd_buf, REMOTE_PORT ); 
    
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QIOPEN, cmd_buf );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  
    Delay_ms( 2000 );
    
    #define LTEIOT12_UDP_SOCKET_OPEN                "QIOPEN: 1,0"
    lteiot12_rsp_check( &lteiot12, LTEIOT12_UDP_SOCKET_OPEN  );
    
    if ( LTEIOT12_OK == app_error_flag )
    {
        log_info( &logger, "SENDING MESSAGES TO UDP" );
        // Write message to UDP socket
        strcpy( cmd_buf, UDP_PROTOCOL );
        strcat( cmd_buf, "," );
        strcat( cmd_buf, message_len_buf );
        lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QISEND, cmd_buf );
        lteiot12_rsp_check( &lteiot12, ">"  );
        strcpy( cmd_buf, MESSAGE_CONTENT );
        strcat( cmd_buf, "\032" );
        
        lteiot12_send_cmd( &lteiot12, cmd_buf );
        
        lteiot12_rsp_check( &lteiot12, "SEND OK"  );
    
        // Read response message from UDP socket
        for ( ; ; )
        {
            lteiot12_process( &lteiot12 );
            uint8_t * __generic_ptr start_response_buf = strstr( app_buf, URC_READ_SOCKET_DATA );
            if ( start_response_buf )
            {
                log_info( &logger, "RECEIVED MESSAGE" );
                strcpy( cmd_buf, UDP_PROTOCOL );
                strcat( cmd_buf, "," );
                strcat( cmd_buf, "256" );
                lteiot12_clear_app_buf( );
                lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QIRD, cmd_buf );
                app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
                lteiot12_error_check( app_error_flag );  
            
                if ( LTEIOT12_OK == app_error_flag )
                {
                    break;
                }
            }
            if ( timeout_cnt++ > timeout )
            {
                log_error( &logger, "TIMEOUT!!!" );
                break;
            }   
            Delay_ms( 1 );
        }
        timeout_cnt = 0;
        Delay_ms( 2000 );
        
        // Close UDP socket
        lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QICLOSE, UDP_PROTOCOL );
        app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
        lteiot12_error_check( app_error_flag );  
    }
    else
    {
        log_error( &logger, "Not connected to the UDP server" );
    }
    
#elif ( DEMO_EXAMPLE == SMS_EXAMPLE )

    #define CMGF_PDU "+CMGF: 0"
    #define CMGF_TXT "+CMGF: 1"
    lteiot12_send_cmd_check( &lteiot12, LTEIOT12_CMD_CMGF );
    lteiot12_process( &lteiot12 );
    if ( strstr( app_buf, CMGF_PDU ) )
    {
        // Send SMS in PDU mode
        lteiot12_send_sms_pdu ( &lteiot12, SIM_SMSC, PHONE_NUMBER_TO_MESSAGE, MESSAGE_CONTENT );
        app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK );
        lteiot12_error_check( app_error_flag );  
    }
    else if ( strstr( app_buf, CMGF_TXT ) )
    {
        // Send SMS in TXT mode
        lteiot12_send_sms_text_mode( &lteiot12, PHONE_NUMBER_TO_MESSAGE, MESSAGE_CONTENT );
        app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK );
        lteiot12_error_check( app_error_flag );  
    }
    Delay_ms( 10000 );
    Delay_ms( 10000 );
    Delay_ms( 10000 );
    
#elif ( DEMO_EXAMPLE == GNSS_EXAMPLE )
    #define LTEIOT12_NEMA_SENTENCE          "\"GGA\""    
    lteiot12_send_cmd_with_params( &lteiot12, LTEIOT12_CMD_QGPSGNMEA, LTEIOT12_NEMA_SENTENCE );
    lteiot12_process( &lteiot12 );
    gnss_parser_application ( app_buf );
    lteiot12_clear_app_buf ( );
#endif 
    Delay_ms( 5000 );
}

static void lteiot12_check_connection( void )
{
    #define CONNECTED "+CGATT: 1"

    if ( strstr( app_buf, CONNECTED ) != 0 )
    {
        app_connection_status = LTEIOT12_EXAMPLE;
    }
}

static void lteiot12_check_con_to_network( void )
{
    lteiot12_send_cmd( &lteiot12, LTEIOT12_CMD_CSQ );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  
    
    lteiot12_send_cmd_check( &lteiot12, LTEIOT12_CMD_CGATT );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    lteiot12_send_cmd_check( &lteiot12, LTEIOT12_CMD_CEREG );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    lteiot12_send_cmd_check( &lteiot12, LTEIOT12_CMD_COPS );
    app_error_flag = lteiot12_rsp_check( &lteiot12, LTEIOT12_RSP_OK  );
    lteiot12_error_check( app_error_flag );  

    if ( LTEIOT12_CONFIGURE_FOR_EXAMPLE != app_connection_status )
    {
        log_info( &logger, "CONNECTED TO NETWORK" );
    }
}

static void gnss_parser_application ( uint8_t *rsp )
{
   uint8_t element_buf[ 100 ] = { 0 };
    if ( LTEIOT12_OK == lteiot12_parse_gngga( rsp, LTEIOT12_GNGGA_LATITUDE, element_buf ) )
    {
        static uint8_t wait_for_fix_cnt = 0;
        if ( 0 < strlen( element_buf ) > 0 )
        {
            log_printf( &logger, "\r\n Latitude: %.2s degrees, %s minutes \r\n", 
                        element_buf, &element_buf[ 2 ] );
            lteiot12_parse_gngga( rsp, LTEIOT12_GNGGA_LONGITUDE, element_buf );
            log_printf( &logger, " Longitude: %.3s degrees, %s minutes \r\n", 
                        element_buf, &element_buf[ 3 ] );
            memset( element_buf, 0, sizeof( element_buf ) );
            lteiot12_parse_gngga( rsp, LTEIOT12_GNGGA_ALTITUDE, element_buf );
            log_printf( &logger, " Altitude: %s m \r\n", element_buf );
            wait_for_fix_cnt = 0;
        }
        else
        {
            if ( wait_for_fix_cnt % 5 == 0 )
            {
                log_printf( &logger, " Waiting for the position fix...\r\n\n" );
                wait_for_fix_cnt = 0;
            }
            wait_for_fix_cnt++;
        }
        lteiot12_clear_app_buf(  );
    }
}


// ------------------------------------------------------------------------ END
